SystemOrganization addCategory: #'Satellogic-Structure'!
SystemOrganization addCategory: #'Satellogic-DataTypes'!
SystemOrganization addCategory: #'Satellogic-Tests'!
SystemOrganization addCategory: #'Satellogic-Console'!
SystemOrganization addCategory: #'Satellogic-SendingCommands'!
SystemOrganization addCategory: #'Satellogic-ParsingAnswers'!

Error subclass: #ParsingError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

TestCase subclass: #SatellogicTest
	instanceVariableNames: 'testObjectFactory newMessageRecived radioObservers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Tests'!

!SatellogicTest methodsFor: 'Radio messages' stamp: 'TomasRodriguez 10/31/2013 18:25'!
addListener: aSatelliteConsole 
	
	radioObservers add: aSatelliteConsole ! !

!SatellogicTest methodsFor: 'Radio messages' stamp: 'TomasRodriguez 10/30/2013 18:14'!
newMessageRecived
	
	^ newMessageRecived! !

!SatellogicTest methodsFor: 'Radio messages' stamp: 'TomasRodriguez 10/30/2013 18:22'!
send: aByteArray 
	
	newMessageRecived := true! !

!SatellogicTest methodsFor: 'Radio messages' stamp: 'TomasRodriguez 11/1/2013 09:02'!
sendAnswer: aByteArray 
	
	radioObservers do: [ :anObserver | anObserver listen: aByteArray ]! !

!SatellogicTest methodsFor: 'private' stamp: 'TomasRodriguez 10/31/2013 18:25'!
setUp

	testObjectFactory := ObjectFactory new .
	radioObservers := OrderedCollection new.
	newMessageRecived := false ! !

!SatellogicTest methodsFor: 'private' stamp: 'TomasRodriguez 10/29/2013 21:38'!
should: aBlock raiseErrorWithMessage: aMessage

	self should: aBlock  
	raise:  (Error but: MessageNotUnderstood )
	withExceptionDo: [ :anError | self assert: anError messageText = aMessage ]! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:00'!
test001AnAnswerParameterHasANameAndADataType

	| dataType answerParameter |
	dataType := Unsigned8BitInteger new .
	answerParameter := OutputParameter of: Unsigned8BitInteger new named: 'battery_temp' .
	
	self assert: answerParameter name = 'battery_temp' .
	self assert: answerParameter dataType = dataType .! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:00'!
test002CanNotCreateAnAnswerParameterWithAnInvalidName

	self should: [OutputParameter of: Unsigned8BitInteger new named: '' ] 
	raiseErrorWithMessage: 'The answer parameter must have a name'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:00'!
test003CanNotCreateAnAnswerParameterWithADataTypeThatDoesNotBelongToTheSatelliteDataTypes

	self should: [OutputParameter of: String new named: 'un string' ]
	raiseErrorWithMessage: 'The data type does not belong to the satellite'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:00'!
test004AnStructIsACompositionOfAnswerParameters


	| otherAnswerParameter struct anAnswerParameter |
	anAnswerParameter := OutputParameter of: Unsigned8BitInteger new named: 'a number' .
	otherAnswerParameter := OutputParameter of: SatelliteString new named: 'a string' .
	struct := Struct with: (OrderedCollection with: anAnswerParameter with: otherAnswerParameter) .
	
	self assert: struct parameters size = 2 .
	self assert: struct parameters first = anAnswerParameter .
	self assert: struct parameters second = otherAnswerParameter ! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 11/1/2013 10:13'!
test005CanNotCreateAStructWithLessThan2Parameters


	self should: [Struct with: (OrderedCollection new)] 
		raiseErrorWithMessage: 'A struct cannot be empty'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:06'!
test006ACommandDefinitionHasAnIdANameACollectionOfInputParametersAndACommandAnswerDefinition

	
	| output commandDefinition |
	output := OutputParameter of: Unsigned8BitInteger new named: 'un numero'.
	commandDefinition := CommandDefinition withId: 5 name: 'name' input: #() output: output  .
	
	self assert: commandDefinition id = 5 .
	self assert: commandDefinition name = 'name' .
	self assert: commandDefinition input isEmpty .
	self assert: commandDefinition output = output ! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:05'!
test007CanNotCreateACommandWithAnAnswerThatIsNotACommandAnswerDefinition

	self should: [CommandDefinition withId: 5 name: 'name' input: #() output: Unsigned8BitInteger new] 
	raiseErrorWithMessage: 'Invalid output for the command'! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:00'!
test008CanNotCreateAStructWithOtherThanAnswerParameters

	| otherAnswerParameter struct anAnswerParameter |
	anAnswerParameter := OutputParameter of: Unsigned8BitInteger new named: 'a number' .
	
	self should: [Struct with: (OrderedCollection with: anAnswerParameter with: Void new)] 
	raiseErrorWithMessage: 'A Struct can only contain answer parameters from the satellite'! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:03'!
test009CanNotCreateACommandDefinitionWithAnInputThatDoesNotIncludeSatelliteDataTypes

	self should: [CommandDefinition withId: 5 name: 'name' input: (Array with: SatelliteString new with: Void new) output: Void new] 
	raiseErrorWithMessage: 'Invalid input for the command'! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 10:28'!
test010ASubSystemHasAnIdANameAnKnowsTheCommandsThatCanAnswer

	| commands subSystem |
	commands := testObjectFactory validThermalCommands .
	subSystem := SubSystemDefinition withId: 3 name: #THERMAL definingCommands: commands .
	
	self assert: subSystem id = 3 .
	self assert: subSystem name = #THERMAL .
	self assert: subSystem commands size = commands size .
	self assert: (subSystem commands includesAll: commands )! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:03'!
test012CanNotCreateASubSystemWithTwoCommandsWithTheSameNameOrId

	| commandsSameName commandsSameId |
	commandsSameName := testObjectFactory validThermalCommands .
	commandsSameName add: (CommandDefinition withId: 50 name: #name input: #() output: Void new ) .
	commandsSameId := testObjectFactory validThermalCommands .
	commandsSameId add: (CommandDefinition withId: 0 name: #new input: #() output: Void new ) .
	
	self should: [SubSystemDefinition withId: 3 name: #THERMAL definingCommands: commandsSameName ] 
	raiseErrorWithMessage: 'Cannot have two commands with the same name or id' .
	
	self should: [SubSystemDefinition withId: 3 name: #THERMAL definingCommands: commandsSameId ] 
	raiseErrorWithMessage: 'Cannot have two commands with the same name or id'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 10:30'!
test013ASatelliteIsACompositionOfSubSystems

	| subSystems satellite |
	subSystems := testObjectFactory validSubSystemsForASatellite .
	satellite := SatelliteDefinition composedOf: subSystems .
	
	self assert: satellite subSystems size = 2 .
	self assert: (satellite subSystems includesAll: subSystems )! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 12:20'!
test014CanNotCreateASatelliteWithSubSystemsWhichDoesNotDefineTheBasicCommands

	| subSystems |
	subSystems := OrderedCollection with: (testObjectFactory subSystemWithoutBasicCommands) .

	self should: [SatelliteDefinition composedOf: subSystems ] 
	raiseErrorWithMessage: 'Cannot create a satellite with subsystems which does not define the basic commands: "name", "telemetry", "short_telemetry"'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:02'!
test015ACommandCallIsFormedByASatelliteASubSystemACommandAndTheInputParmetersToBeSent

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := satellite subSystems first .
	command := subSystem command: #name .
	commandCall := CommandCall for: command from: subSystem in: satellite withInputParameters: #() .
	
	self assert: commandCall satellite = satellite .
	self assert: commandCall subSystem = subSystem .
	self assert: commandCall command = command .
	self assert: commandCall inputForTheCommand = #() .! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:06'!
test016CanNotCreateACommandCallIfTheSubSystemDoesNotBelongToTheSatellite

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := SubSystemDefinition 
		withId: 10 
		name: #unknown 
		definingCommands: (OrderedCollection with: testObjectFactory aNameCommandDefinition ).
	command := subSystem command: #name .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: #() ]
	raiseErrorWithMessage: 'The subsystem does not belong to the satellite'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/31/2013 18:03'!
test017CanNotCreateACommandCallIfTheCommandDoesNotBelongToTheSubSystem

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := satellite subSystems first .
	command := CommandDefinition withId: 23 name: #unknown input: #() output: Void new .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: #() ]
	raiseErrorWithMessage: 'The subsystem cannot answer the command'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:54'!
test018CanNotCreateACommandCallIfTheInputParametersSizeIsNotTheSameThatTheCommandDefinitionInputSize

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := satellite subSystems first .
	command := subSystem command: #name .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: #(23) ]
	raiseErrorWithMessage: 'The input parameter does not have the correct number of arguments for the command'
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:32'!
test019AnUnsignedIntegerCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| unsignedIntegers |
	unsignedIntegers := OrderedCollection new .
	unsignedIntegers 
		add: Unsigned8BitInteger new;
		add: Unsigned16BitInteger new;
		add: Unsigned24BitInteger new;
		add: Unsigned32BitInteger new;
		add: Unsigned64BitInteger new;
		yourself .
	
	unsignedIntegers do: [ :anUnsignedInteger | | exponent |
		exponent := anUnsignedInteger bitsUsed .
		self assert: (anUnsignedInteger canBeRepresentedWith: 1) .
		self deny: (anUnsignedInteger canBeRepresentedWith: -1) .
		self deny: (anUnsignedInteger canBeRepresentedWith: (2 raisedTo: exponent ) ) .
		self deny: (anUnsignedInteger canBeRepresentedWith: 'hola')
		]
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:34'!
test020ASignedIntegerCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| signedIntegers |
	signedIntegers := OrderedCollection new .
	signedIntegers 
		add: Signed8BitInteger new;
		add: Signed16BitInteger new;
		add: Signed32BitInteger new;
		add: Signed64BitInteger new;
		yourself .
	
	signedIntegers do: [ :aSignedInteger | | exponent |
		exponent := aSignedInteger bitsUsed - 1 .
		self assert: (aSignedInteger canBeRepresentedWith: 1) .
		self assert: (aSignedInteger canBeRepresentedWith: -1) .
		self deny: (aSignedInteger canBeRepresentedWith: (2 raisedTo: exponent ) ) .
		self deny: (aSignedInteger canBeRepresentedWith: ((2 raisedTo: exponent ) + 1) negated ) .
		self deny: (aSignedInteger canBeRepresentedWith: 'hola')
		]
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:38'!
test021ASatelliteStringCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| satelliteString |
	satelliteString := SatelliteString new .
	
	self assert: ( satelliteString canBeRepresentedWith: 'Hola' ) .
	self deny: ( satelliteString canBeRepresentedWith: 872 ) .
! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:43'!
test022ASizedStringCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| sizedStrings |
	sizedStrings := OrderedCollection new .
	sizedStrings 
		add: String8 new;
		add: String16 new;
		yourself .
	
	sizedStrings do: [ :aSizedString | | stringSize |
		
		stringSize := aSizedString sizeRepresentation bitsUsed .
		self assert: (aSizedString canBeRepresentedWith: 'asdasd') .
		self deny: (aSizedString canBeRepresentedWith: 123 ) .
		self deny: (aSizedString canBeRepresentedWith: (String new: (2 raisedTo: stringSize)))
		]! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:48'!
test023AnArrayOfFixedSizeCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| arrayOfFixedSize |
	arrayOfFixedSize := ArrayOfFixedSize of: Unsigned64BitInteger new withSize: 5 .
	
	self assert: (arrayOfFixedSize canBeRepresentedWith: #(2 3 4 5 6) ) .
	self deny: (arrayOfFixedSize canBeRepresentedWith: #(2 3 5 6) ) .
	self deny: (arrayOfFixedSize canBeRepresentedWith: #('hola' 'hola' 'hola' 'hola' 'hola') ) .! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:51'!
test024AnEnumCanAnswerIfAParameterIsACorrectRepresentationOfIt

	| enumeration |
	enumeration := Enum withLabels: #(ON OFF).
	
	self assert: (enumeration canBeRepresentedWith: #ON ) .
	self assert: (enumeration canBeRepresentedWith: #OFF ) .
	self deny: (enumeration canBeRepresentedWith: #UNKNOWN ) .! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 10/30/2013 11:59'!
test025CanNotCreateACommandCallIfTheInputParametersDoesNotFitTheCommandDefinitionInput

	| satellite commandCall subSystem command |
	satellite := testObjectFactory validSatelliteDefinition .
	subSystem := satellite subSystems first .
	command := subSystem command: #setMatrixKeyKey: .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: #('hola') ]
	raiseErrorWithMessage: 'The input parameter does not fit the command input' .
	
	self should: [ CommandCall for: command from: subSystem in: satellite withInputParameters: (Array with: #(23)) ]
	raiseErrorWithMessage: 'The input parameter does not fit the command input' .
	
	CommandCall for: command from: subSystem in: satellite withInputParameters: #(23)
! !

!SatellogicTest methodsFor: 'Tests 02 Sending Commands' stamp: 'TomasRodriguez 10/30/2013 17:46'!
test026ACommandSerializerCanSerializeACommandCallWithoutInputParameters

	| commandSerializer |
	commandSerializer := CommandSerializer for: testObjectFactory validThermalNameCommandCall .
	
	self assert: commandSerializer value = #[3 0]! !

!SatellogicTest methodsFor: 'Tests 02 Sending Commands' stamp: 'TomasRodriguez 10/30/2013 17:46'!
test027ACommandSerializerCanSerializeACommandCallWithInputParameters

	| commandSerializer |
	commandSerializer := CommandSerializer for: (testObjectFactory validThermalSetMatrixKeyKeyCommandCallWithInput: #(70)) .
	
	self assert: commandSerializer value = #[3 32 0 0 0 70]! !

!SatellogicTest methodsFor: 'Tests 02 Sending Commands' stamp: 'TomasRodriguez 10/30/2013 17:42'!
test028ASerializationFormulasHasTheCorrectSerializationMethodForTheSatelliteDataTypes
	
	| serializationFormulas |
	serializationFormulas := SerializationFormulas new .
	
	self assert: ((serializationFormulas lookFor: Unsigned8BitInteger new) value: 23) = #[23] .
	self assert: ((serializationFormulas lookFor: Unsigned16BitInteger new) value: 23) = #[0 23] .
	self assert: ((serializationFormulas lookFor: Unsigned24BitInteger new) value: 23) = #[0 0 23] .
	self assert: ((serializationFormulas lookFor: Unsigned32BitInteger new) value: 23) = #[0 0 0 23] .
	self assert: ((serializationFormulas lookFor: Unsigned64BitInteger new) value: 23) = #[0 0 0 0 0 0 0 23] .
	
	self assert: ((serializationFormulas lookFor: Signed8BitInteger new) value: -1) = #[255] .
	self assert: ((serializationFormulas lookFor: Signed16BitInteger new) value: -1) = #[255 255] .
	self assert: ((serializationFormulas lookFor: Signed32BitInteger new) value: -1) = #[255 255 255 255] .
	self assert: ((serializationFormulas lookFor: Signed64BitInteger new) value: -1) = #[255 255 255 255 255 255 255 255] .
	
	self assert: ((serializationFormulas lookFor: SatelliteString new) value: 'PEPITO') = #[80 69 80 73 84 79] .
	
	self assert: ((serializationFormulas lookFor: String8 new) value: 'PEPITO') = #[6 80 69 80 73 84 79] .
	self assert: ((serializationFormulas lookFor: String16 new) value: 'PEPITO') = #[0 6 80 69 80 73 84 79] .
	
	self assert: ((serializationFormulas lookFor: (Enum withLabels: #(ON OFF))) value: 'ON') = #[0] .
	self assert: ((serializationFormulas lookFor: (Enum withLabels: #(ON OFF))) value: 'OFF') = #[1] .
	
	self assert: ((serializationFormulas lookFor: (ArrayOfFixedSize of: Unsigned24BitInteger new withSize: 3 )) value: #(1 2 3) ) = #[0 0 1 0 0 2 0 0 3] .
	self assert: ((serializationFormulas lookFor: (ArrayOfFixedSize of: String8 new withSize: 2 )) value: #('PEPITO' 'PEPITO') ) = #[6 80 69 80 73 84 79 6 80 69 80 73 84 79] .! !

!SatellogicTest methodsFor: 'Tests 02 Sending Commands' stamp: 'TomasRodriguez 10/30/2013 18:11'!
test029ACommandSenderNextSequenceNumberIsOneWhenCreated 

	| commandSender |
	commandSender := CommandSender sendThrough: self .
	
	self assert: commandSender nextSequenceNumber = 16r1! !

!SatellogicTest methodsFor: 'Tests 02 Sending Commands' stamp: 'TomasRodriguez 10/30/2013 18:25'!
test030ACommandSenderCanSendACommandThroughTheRadioAndAfterThatTheSequenceNumberIsIncrementedByOne

	| commandSender |
	commandSender := CommandSender sendThrough: self .
	
	self deny: self newMessageRecived .
	commandSender send: testObjectFactory validThermalNameCommandCall .
	self assert: commandSender nextSequenceNumber = 16r2 .
	self assert: self newMessageRecived ! !

!SatellogicTest methodsFor: 'Tests 02 Sending Commands' stamp: 'TomasRodriguez 10/30/2013 18:26'!
test031ACommandSenderDoWrapAroundWhenTheSequenceNumberBecomesGreaterThat16rFFFFF0

	| commandSender |
	commandSender := MockCommandSender sendThrough: self withSequenceNumber: 16rFFFFF0.
	
	self assert: commandSender nextSequenceNumber = 16rFFFFF0 .
	commandSender send: testObjectFactory validThermalNameCommandCall .
	self assert: commandSender nextSequenceNumber = 16r1 .! !

!SatellogicTest methodsFor: 'Tests 02 Sending Commands' stamp: 'TomasRodriguez 11/1/2013 10:42'!
test032ACommandSenderReturnsACommandTicketWithTheInformationOfTheCommandSent

	| commandSender commandCall commandTicket |
	commandSender := CommandSender sendThrough: self .
	commandCall := testObjectFactory validThermalNameCommandCall .
	commandTicket := commandSender send: commandCall .
	
	self assert: commandTicket commandSent = commandCall .
	self assert: commandTicket sequenceNumber = 16r1 .
	self assert: commandTicket package = #[215 221 48 0 0 1 3 0] .! !

!SatellogicTest methodsFor: 'Tests 03 Receiving Answers' stamp: 'TomasRodriguez 10/31/2013 18:11'!
test033ASatelliteAnswerCanDistinguishTheDifferentTypesOfAnswersOfTheSatellite

	self assert: ((SatelliteAnswer for: #[255 255 240]) isKindOf: Beacon) .
	self assert: ((SatelliteAnswer for: #[255 255 243]) isKindOf: ShortBeacon) .
	self assert: ((SatelliteAnswer for: #[0 0 1]) isKindOf: CommandAnswer) .
	self assert: ((SatelliteAnswer for: #[255 255 255]) isKindOf: UnknownAnswer) ! !

!SatellogicTest methodsFor: 'Tests 03 Receiving Answers' stamp: 'TomasRodriguez 10/31/2013 08:48'!
test034AParsingFormulasHasTheCorrectParsingMethodForTheSatelliteDataTypes
	
	| parsingFormulas |
	parsingFormulas := ParsingFormulas new .
	
	self assert: ((parsingFormulas lookFor: Unsigned8BitInteger new) value: (ReadStream on: #[23])) = 23 .
	self assert: ((parsingFormulas lookFor: Unsigned16BitInteger new) value: (ReadStream on: #[0 23])) = 23 .
	self assert: ((parsingFormulas lookFor: Unsigned24BitInteger new) value: (ReadStream on: #[0 0 23])) = 23 .
	self assert: ((parsingFormulas lookFor: Unsigned32BitInteger new) value: (ReadStream on: #[0 0 0 23])) = 23 .
	self assert: ((parsingFormulas lookFor: Unsigned64BitInteger new) value: (ReadStream on: #[0 0 0 0 0 0 0 23])) = 23 .
	
	self assert: ((parsingFormulas lookFor: Signed8BitInteger new) value: (ReadStream on: #[255])) = -1 .
	self assert: ((parsingFormulas lookFor: Signed16BitInteger new) value: (ReadStream on: #[255 255])) = -1 .
	self assert: ((parsingFormulas lookFor: Signed32BitInteger new) value: (ReadStream on: #[255 255 255 255])) = -1 .
	self assert: ((parsingFormulas lookFor: Signed64BitInteger new) value: (ReadStream on: #[255 255 255 255 255 255 255 255])) = -1 .
	
	self assert: ((parsingFormulas lookFor: SatelliteString new) value: (ReadStream on: #[80 69 80 73 84 79])) = 'PEPITO' .
	
	self assert: ((parsingFormulas lookFor: String8 new) value: (ReadStream on: #[6 80 69 80 73 84 79])) = 'PEPITO'.
	self assert: ((parsingFormulas lookFor: String16 new) value:(ReadStream on: #[0 6 80 69 80 73 84 79])) = 'PEPITO' .
	
	self assert: ((parsingFormulas lookFor: (Enum withLabels: #(ON OFF))) value: (ReadStream on: #[0])) = #ON .
	self assert: ((parsingFormulas lookFor: (Enum withLabels: #(ON OFF))) value: (ReadStream on: #[1])) = #OFF .
	
	self assert: ((parsingFormulas lookFor: (ArrayOfFixedSize of: Unsigned24BitInteger new withSize: 3 )) value: (ReadStream on: #[0 0 1 0 0 2 0 0 3]) ) = #(1 2 3) .
	self assert: ((parsingFormulas lookFor: (ArrayOfFixedSize of: String8 new withSize: 2 )) value: (ReadStream on: #[6 80 69 80 73 84 79 6 80 69 80 73 84 79])) = #('PEPITO' 'PEPITO')! !

!SatellogicTest methodsFor: 'Tests 03 Receiving Answers' stamp: 'TomasRodriguez 10/31/2013 20:40'!
test035ACommandOutputParserCanParseAnAnswerParameterAndReturnsAParsedAnswerParameter
	
	| answerParameter parser parsedParameter |
	answerParameter := OutputParameter of: Unsigned32BitInteger new named: 'un numero' .
	parser := CommandOutputParser for: answerParameter as: (ReadStream on: #[0 0 0 1] ).
	parsedParameter := parser value .
	
	self assert: parsedParameter name = 'un numero' .
	self assert: parsedParameter value = 1.! !

!SatellogicTest methodsFor: 'Tests 03 Receiving Answers' stamp: 'TomasRodriguez 10/31/2013 20:39'!
test036ACommandOutputParserCanParseAStructAndReturnsACollectionOfParsedAnswerParameter
	
	| answer struct parser parsedParameters |
	answer := OrderedCollection new .
	answer add: (OutputParameter of: Signed16BitInteger new named: 'battery_temp' ) .
	answer add: (OutputParameter of: Signed16BitInteger new named: 'radio_temp' ) .
	struct := Struct with: answer .
	parser := CommandOutputParser for: struct as: (ReadStream on: #[255 255 0 1] ).
	parsedParameters := (parser value) parsedParameters .

	self assert: parsedParameters size = 2.
	self assert: parsedParameters first = (ParsedOutputParameter with: -1 named: 'battery_temp') .
	self assert: parsedParameters second = (ParsedOutputParameter with: 1 named: 'radio_temp') .! !

!SatellogicTest methodsFor: 'Tests 03 Receiving Answers' stamp: 'TomasRodriguez 10/31/2013 20:39'!
test037ACommandOutputParserCanParseAStructAndReturnsAnEmptyAnswer
	
	| parser parsedParameter |
	parser := CommandOutputParser for: Void new as: (ReadStream on: #[255 255 0 1] ).
	parsedParameter := parser value .

	self assert: parsedParameter = EmptyAnswer new .! !

!SatellogicTest methodsFor: 'Tests 03 Receiving Answers' stamp: 'TomasRodriguez 10/31/2013 20:39'!
test038ACommandOutputParserCanParseAnArrayUpToEndOfPackageAndReturnsAnAnswerParameter
	
	| parser parsedParameter |
	parser := CommandOutputParser for: (ArrayUpToEndOfPackage of: Unsigned24BitInteger new named: 'arraycito') as: (ReadStream on: #[0 0 1 0 0 2 0 0 9 0 0 5 0 0 7] ).
	parsedParameter := parser value .

	self assert: parsedParameter name = 'arraycito' .
	self assert: parsedParameter value size = 5 .
	self assert: (parsedParameter value includesAll: #(1 2 9 5 7))! !

!SatellogicTest methodsFor: 'Tests 03 Receiving Answers' stamp: 'TomasRodriguez 10/31/2013 21:13'!
test039ACommandAnswerParserCanParseCommandAnswerSentToTheSatellite
	
	| parser parsedAnswerForACommand |
	"parsea un comando con una respuesta simple"
	parser := CommandAnswerParser for: (SatelliteAnswer for: #[0 0 1 84 72 69 82 77 65 76]) from: testObjectFactory validThermalNameCommandCall .
	parsedAnswerForACommand := parser value .
	
	self assert: parsedAnswerForACommand name = 'name' .
	self assert: parsedAnswerForACommand value = 'THERMAL' .
	
	"parsea un comando con una struct de respuesta"
	parser := CommandAnswerParser for: (SatelliteAnswer for: #[0 0 1 255 255 0 1]) from: testObjectFactory validThermalTelemetryCommandCall .
	parsedAnswerForACommand := parser value .
	
	self assert: parsedAnswerForACommand parsedParameters size = 2.
	self assert: parsedAnswerForACommand parsedParameters first = (ParsedOutputParameter with: -1 named: 'battery_temp') .
	self assert: parsedAnswerForACommand parsedParameters second = (ParsedOutputParameter with: 1 named: 'radio_temp') .! !

!SatellogicTest methodsFor: 'Tests 03 Receiving Answers' stamp: 'TomasRodriguez 10/31/2013 18:11'!
test040ASignalParserCanParseABeaconSentFromASatellite
	
	| parser parsedBeacon parsedThermalTelemetry |
	parser := SignalParser for: testObjectFactory validBeaconAnswer from: testObjectFactory validSatelliteDefinition .
	parsedBeacon := parser value .
	
	self assert: (parsedBeacon informationOf: testObjectFactory aThermalSubSystemDefinition) parsedParameters first = (ParsedOutputParameter with: -1 named: 'battery_temp' ).
	self assert: (parsedBeacon informationOf: testObjectFactory aThermalSubSystemDefinition) parsedParameters second = (ParsedOutputParameter with: 1 named: 'radio_temp' ).
	self assert: (parsedBeacon informationOf: testObjectFactory aPlatformSubSystemDefinition) parsedParameters first = (ParsedOutputParameter with: 25 named: 'tickCount' ).
	self assert: (parsedBeacon informationOf: testObjectFactory aPlatformSubSystemDefinition) parsedParameters second = (ParsedOutputParameter with: 90 named: 'currentMode' ).! !

!SatellogicTest methodsFor: 'Tests 03 Receiving Answers' stamp: 'TomasRodriguez 10/31/2013 18:11'!
test041ASignalParserCanParseAShortBeaconSentFromASatellite
	
	| parser parsedBeacon parsedThermalTelemetry |
	parser := SignalParser for: testObjectFactory validShortBeaconAnswer from: testObjectFactory validSatelliteDefinition .
	parsedBeacon := parser value .
	
	self assert: (parsedBeacon informationOf: testObjectFactory aThermalSubSystemDefinition) = (ParsedOutputParameter with: -1 named: 'battery_temp' ).
	self assert: (parsedBeacon informationOf: testObjectFactory aPlatformSubSystemDefinition) = EmptyAnswer new.! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 10/31/2013 11:56'!
test042ASatelliteConsoleIsTurnedOffWhenCreated

	| satelliteDefinition console |
	satelliteDefinition := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satelliteDefinition usingRadio: self .
	
	self assert: console satellite = satelliteDefinition .
	self assert: console state = TurnedOffState new .! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 10/31/2013 11:58'!
test043ASatelliteConsoleCanBeTurnedOn

	| satelliteDefinition console |
	satelliteDefinition := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satelliteDefinition usingRadio: self .
	console turnOn .
	
	self assert: console state = TurnedOnState new! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 11/1/2013 10:42'!
test044ASatelliteConsoleCanSendACommandToTheSatelliteThroughTheRadio

	| satelliteDefinition console commandTicket commandCall |
	satelliteDefinition := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satelliteDefinition usingRadio: self .
	console turnOn .
	commandCall := testObjectFactory validThermalNameCommandCall .
	
	self deny: self newMessageRecived .
	commandTicket := console send: commandCall .
	self assert: self newMessageRecived .
	self assert: commandTicket commandSent = commandCall! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 10/31/2013 12:12'!
test045CanNotSendACommandIfTheConsoleIsTurnedOff

	| satelliteDefinition console commandTicket commandCall |
	satelliteDefinition := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satelliteDefinition usingRadio: self .
	commandCall := testObjectFactory validThermalNameCommandCall .
	
	self deny: self newMessageRecived .
	self should: [console send: commandCall ]
	raise: (Error but: MessageNotUnderstood )
	withExceptionDo: [ :anError |
		self assert: anError messageText = 'The console is turned off' .
		self deny: self newMessageRecived .
		] ! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 10/31/2013 12:18'!
test046ASatelliteConsoleHasNoAnswerRecordsWhenCreated

	| satelliteDefinition console commandTicket commandCall |
	satelliteDefinition := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satelliteDefinition usingRadio: self .
	
	self assert: console records isEmpty ! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 10/31/2013 21:20'!
test047ASignalRecordHasTheParsedSignalAndThePackageReceivedFromTheSatellite

	| parsedBeacon satelliteAnswer record |
	parsedBeacon := testObjectFactory validParsedBeacon .
	satelliteAnswer := testObjectFactory validBeaconAnswer .
	record := SignalRecord of: parsedBeacon fromSatelliteAnswer: satelliteAnswer .
	
	self assert: record parsedAnswer = parsedBeacon .
	self assert: record satelliteAnswer = satelliteAnswer .! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 10/31/2013 21:27'!
test048ACommandAnswerRecordHasTheParsedCommandAnswerAndTheCommandTicketAssociated

	| parsedCommandAnswer satelliteAnswer commandCall record |
	satelliteAnswer := testObjectFactory validThermalNameSatelliteAnswer .
	commandCall := testObjectFactory validThermalNameCommandCall .
	parsedCommandAnswer := (CommandAnswerParser for: satelliteAnswer from: commandCall ) value .
	record := CommandAnswerRecord of: parsedCommandAnswer fromSatelliteAnswer: satelliteAnswer withCommandCall: commandCall .
	
	self assert: record parsedAnswer = parsedCommandAnswer .
	self assert: record satelliteAnswer  = satelliteAnswer .
	self assert: record commandCall = commandCall ! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 11/1/2013 09:23'!
test049ASatelliteConsoleRecordASatelliteAnswerWhenItIsHeardByTheRadio

	| beacon shortBeacon satellite console |
	beacon := testObjectFactory validBeaconAnswer .
	shortBeacon := testObjectFactory validShortBeaconAnswer .
	satellite := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satellite usingRadio: self .
	console turnOn .
	self sendAnswer: beacon package .
	self sendAnswer: shortBeacon package .
	
	self assert: console records size = 2 .
	self assert: console records second satelliteAnswer = beacon .
	self assert: console records second parsedAnswer = (SignalParser for: beacon from: satellite ) value .
	self assert: console records first satelliteAnswer = shortBeacon .
	self assert: console records first parsedAnswer = (SignalParser for: shortBeacon from: satellite ) value! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 11/1/2013 09:25'!
test050ASatelliteConsoleDoesNotListenToTheRadioIfItIsTurnedOff

	| beacon shortBeacon satellite console |
	beacon := testObjectFactory validBeaconAnswer .
	satellite := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satellite usingRadio: self .

	self sendAnswer: beacon package .
	
	self assert: console records isEmpty ! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 11/1/2013 09:46'!
test051ASatelliteConsoleRecordsACommandAnswerAfterTheCommandWasSent

	| commandCall satellite console satelliteAnswer |
	commandCall := testObjectFactory validThermalNameCommandCall .
	satellite := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satellite usingRadio: self .
	console turnOn .
	console send: commandCall .
	self sendAnswer: #[0 0 1 84 72 69 82 77 65 76] .
	
	satelliteAnswer := SatelliteAnswer for: #[0 0 1 84 72 69 82 77 65 76] .
	self assert: console records size = 1 .
	self assert: console records first satelliteAnswer = satelliteAnswer .
	self assert: console records first parsedAnswer = (CommandAnswerParser for: satelliteAnswer from: commandCall) value .! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 11/1/2013 09:53'!
test052ASatelliteConsoleDoesNotRecordACommandAnswerForACommandThatWasNotSent

	| commandCall satellite console satelliteAnswer |
	commandCall := testObjectFactory validThermalNameCommandCall .
	satellite := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satellite usingRadio: self .
	console turnOn .
	self sendAnswer: #[0 0 1 84 72 69 82 77 65 76] .

	self assert: console records isEmpty .! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 11/1/2013 10:20'!
test053ACompoundCommandIsACompositionOfCommandCalls
	
	| commands compoundCommand |
	commands := OrderedCollection new .
	commands add: (testObjectFactory validThermalNameCommandCall ) .
	commands add: (testObjectFactory validThermalSetMatrixKeyKeyCommandCallWithInput: #(2) ) .
	compoundCommand := CompoundCommandCall with: commands .
	
	self assert: compoundCommand commandCalls = commands! !

!SatellogicTest methodsFor: 'Tests 01 Structure' stamp: 'TomasRodriguez 11/1/2013 10:22'!
test054CanNotCreateACompundCommandCallWithNoCommandCalls
	
	self should: [CompoundCommandCall with: (OrderedCollection new) ] 
	raiseErrorWithMessage: 'A compound command cannot be created without commands'! !

!SatellogicTest methodsFor: 'Tests 02 Sending Commands' stamp: 'TomasRodriguez 11/1/2013 10:25'!
test055ACompoundCommandCallCanBeSerialized
	
	| commands compoundCommand |
	commands := OrderedCollection new .
	commands add: (testObjectFactory validThermalNameCommandCall ) .
	commands add: (testObjectFactory validThermalSetMatrixKeyKeyCommandCallWithInput: #(2) ) .
	compoundCommand := CompoundCommandCall with: commands .
	
	self assert: (CommandSerializer for: compoundCommand ) value = #[3 0 3 32 0 0 0 2]! !

!SatellogicTest methodsFor: 'Tests 04 Satellite Console' stamp: 'TomasRodriguez 11/1/2013 10:47'!
test056ASatelliteConsoleSendTheCommandsToTheRadioWithAnIncresedSequenceNumber
	
	| satelliteDefinition console commandTicket commandCall |
	satelliteDefinition := testObjectFactory validSatelliteDefinition .
	console := SatelliteConsole for: satelliteDefinition usingRadio: self .
	console turnOn .
	commandCall := testObjectFactory validThermalNameCommandCall .
	
	console send: commandCall .
	commandTicket := console send: commandCall .
	
	self assert: commandTicket sequenceNumber = 16r2! !

Object subclass: #CommandCall
	instanceVariableNames: 'command subSystem satellite inputForTheCommand'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:12'!
commandMissingMessage
	
	^ 'The subsystem cannot answer the command'! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:00'!
for: aCommandDefinition from: aSubSystemDefinition in: aSatelliteDefinition withInputParameters: anOrderedCollection

	self verifyThat: aSubSystemDefinition isComposedIn: aSatelliteDefinition.
	self verifyThat: aCommandDefinition isDefinedIn: aSubSystemDefinition.
	self verifyThat: anOrderedCollection fitsTheInputOf: aCommandDefinition.
	
	^ self new
		initializeWithCommandDefinition: aCommandDefinition
		withSubSystemDefinition: aSubSystemDefinition
		withSatelliteDefinition: aSatelliteDefinition
		withInputForTheCommand: anOrderedCollection! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:58'!
invalidInputMessage
	
	^ 'The input parameter does not fit the command input'! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:21'!
invalidNumberOfInputsMessage
	
	^ 'The input parameter does not have the correct number of arguments for the command'! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:08'!
subSystemMissingMessage
	
	^ 'The subsystem does not belong to the satellite'! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:02'!
verifyThat: aCommandInput fitsTheInputOf: aCommandDefinition
	
	| stream |		
	stream := ReadStream on: aCommandInput .
	
	(aCommandDefinition input allSatisfy: [ :anInput | anInput canBeRepresentedWith: stream next ])
		ifFalse: [ self error: self invalidInputMessage ] .
	
	stream atEnd ifFalse: [ self error: self invalidNumberOfInputsMessage ].! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:26'!
verifyThat: aSubSystemDefinition isComposedIn: aSatelliteDefinition

	(aSatelliteDefinition subSystems includes: aSubSystemDefinition)
		ifFalse: [ self error: self subSystemMissingMessage ]! !

!CommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:15'!
verifyThat: aCommandDefinition isDefinedIn: aSubSystemDefinition
	(aSubSystemDefinition commands includes: aCommandDefinition)
		ifFalse: [ self error: self commandMissingMessage ]! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:32'!
accept: aVisitor
	
	^ aVisitor visitCommandCall: self ! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:03'!
command

	^ command! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:02'!
initializeWithCommandDefinition: aCommandDefinition withSubSystemDefinition: aSubSystemDefinition withSatelliteDefinition: aSatelliteDefinition withInputForTheCommand: anOrderedCollection 

	command := aCommandDefinition .
	subSystem := aSubSystemDefinition .
	satellite := aSatelliteDefinition .
	inputForTheCommand := anOrderedCollection ! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:03'!
inputForTheCommand

	^ inputForTheCommand! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:02'!
satellite

	^ satellite! !

!CommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:03'!
subSystem

	^ subSystem! !

Object subclass: #CommandOutput
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!CommandOutput methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:30'!
= aCommandAnswerDefinition 

	self class = aCommandAnswerDefinition class ifTrue: [ ^ self compareWith: aCommandAnswerDefinition ] .
	^ false! !

!CommandOutput methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:25'!
accept: aVisitor

	self subclassResponsibility ! !

!CommandOutput methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:31'!
compareWith: aCommandAnswerDefinition 

	self subclassResponsibility ! !

CommandOutput subclass: #SimpleOutput
	instanceVariableNames: 'dataType name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

SimpleOutput subclass: #ArrayUpToEndOfPackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!ArrayUpToEndOfPackage methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:30'!
accept: aVisitor 
	
	^ aVisitor visitArrayUpToEndOfPackage: self! !

SimpleOutput subclass: #OutputParameter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!OutputParameter methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:10'!
accept: aVisitor 
	
	^ aVisitor visitOutputParameter: self! !

!SimpleOutput class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:28'!
invalidDataTypeMessage
	
	^ 'The data type does not belong to the satellite'! !

!SimpleOutput class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:28'!
invalidNameMessage
	
	^ 'The answer parameter must have a name'! !

!SimpleOutput class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:28'!
of: aDataType named: aString 
	
	(aDataType isKindOf: SatelliteDataType) ifFalse: [ self error: self invalidDataTypeMessage ] .
	aString isEmpty ifTrue: [ self error: self invalidNameMessage ] .
	
	^ self new initializeWithDataType: aDataType andName: aString ! !

!SimpleOutput methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:29'!
compareWith: anAnswerParameter 

	^ self dataType = anAnswerParameter dataType and:
		( self name = anAnswerParameter name)! !

!SimpleOutput methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:29'!
dataType 

	^ dataType ! !

!SimpleOutput methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:28'!
initializeWithDataType: aDataType andName: aString 

	name := aString .
	dataType := aDataType ! !

!SimpleOutput methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:29'!
name

	^ name! !

CommandOutput subclass: #Struct
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!Struct class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 08:50'!
invalidAnswerParameterMessage
	
	^ 'A Struct can only contain answer parameters from the satellite'! !

!Struct class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:14'!
invalidParameterSizeMessage
	
	^ 'A struct cannot be empty'! !

!Struct class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:13'!
with: anOrderedCollection 
	
	anOrderedCollection isEmpty ifTrue: [ self error: self invalidParameterSizeMessage ]. 
	(anOrderedCollection allSatisfy: [ :anElement | anElement isKindOf: OutputParameter ])
		ifFalse: [ self error: self invalidAnswerParameterMessage ].
	
	^ self new initializeWithAnswerParameters: anOrderedCollection ! !

!Struct methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:09'!
accept: aVisitor 
	
	^ aVisitor visitStruct: self! !

!Struct methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:30'!
compareWith: aStruct

	^ self parameters = aStruct parameters ! !

!Struct methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:32'!
initializeWithAnswerParameters: anOrderedCollection 
	
	parameters := anOrderedCollection ! !

!Struct methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:32'!
parameters

	^ parameters! !

CommandOutput subclass: #Void
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!Void methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:22'!
accept: aVisitor
	
	^ aVisitor visitVoid: self! !

!Void methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:30'!
compareWith: aVoid

	^ true! !

Object subclass: #CommandOutputParser
	instanceVariableNames: 'stream satelliteAnswer parsedAnswer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!CommandOutputParser class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:03'!
for: aSatelliteAnswer as: aStream 

	^ self new initializeWithSatelliteAnswer: aSatelliteAnswer withStreamToParse: aStream .! !

!CommandOutputParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:03'!
initializeWithSatelliteAnswer: aSatelliteAnswer withStreamToParse: aReadStream 
	
	satelliteAnswer := aSatelliteAnswer .
	stream := aReadStream ! !

!CommandOutputParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:17'!
parse
	
	parsedAnswer isNil ifTrue: [ parsedAnswer := satelliteAnswer accept: self ] .
	^ parsedAnswer ! !

!CommandOutputParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 10:07'!
value
	
	parsedAnswer isNil ifTrue: [ parsedAnswer := satelliteAnswer accept: self ] .
	^ parsedAnswer ! !

!CommandOutputParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:11'!
visitArrayUpToEndOfPackage: anArrayUpToEndOfPackage 
	
	| parsedValue |
	parsedValue := OrderedCollection new .
	[ stream atEnd ] 
		whileFalse: [ parsedValue add: ((ParsingFormulas new lookFor: anArrayUpToEndOfPackage dataType) value: stream) ] .
	^ ParsedOutputParameter with: parsedValue named: anArrayUpToEndOfPackage name .! !

!CommandOutputParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:11'!
visitOutputParameter: anAnswerParameter 
	
	| parsedValue |
	parsedValue := (ParsingFormulas new lookFor: anAnswerParameter dataType) value: stream .
	^ ParsedOutputParameter with: parsedValue named: anAnswerParameter name .! !

!CommandOutputParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:09'!
visitStruct: aStruct 
	
	| answerParameters |
	answerParameters := aStruct parameters collect: [ :anAnswerParameter |
		(CommandOutputParser for: anAnswerParameter as: stream) value .
		] .
	^ ParsedStruct with: answerParameters ! !

!CommandOutputParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:22'!
visitVoid: aVoid 
	
	^ EmptyAnswer new! !

Object subclass: #CommandSender
	instanceVariableNames: 'radio nextSequenceNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-SendingCommands'!

!CommandSender class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 18:11'!
sendThrough: aRadio
	
	^ self new initializeWithRadio: aRadio ! !

!CommandSender methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 18:28'!
incrementSequenceNumber
	
	nextSequenceNumber < 16rFFFF0 
		ifTrue: [ nextSequenceNumber := nextSequenceNumber + 16r1] 
		ifFalse: [ nextSequenceNumber := 16r1 ]! !

!CommandSender methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 18:12'!
initializeWithRadio: aRadio

	radio := aRadio .
	nextSequenceNumber := 16r1! !

!CommandSender methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 18:12'!
nextSequenceNumber

	^ nextSequenceNumber! !

!CommandSender methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:40'!
send: aCommandToSend

	| serializedCommand |
	serializedCommand := self serializeCommandFrom: aCommandToSend .
	radio send: serializedCommand .
	self incrementSequenceNumber .
	^ CommandTicket for: aCommandToSend as: serializedCommand 
	
	! !

!CommandSender methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 19:00'!
serializeCommandFrom: aCommandCall
	
	| serializedCommandCall serializedCommand serializedSequenceNumber serializedMac |
	serializedCommandCall := (CommandSerializer for: aCommandCall) value.
	serializedSequenceNumber := (SerializationFormulas new lookFor: Unsigned24BitInteger new)	value: self nextSequenceNumber.
	serializedMac := (MD5MacGenerator for: serializedCommandCall with: self nextSequenceNumber) value.
	^ serializedMac , serializedSequenceNumber , serializedCommandCall.
	! !

CommandSender subclass: #MockCommandSender
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Tests'!

!MockCommandSender class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 18:27'!
sendThrough: aRadio withSequenceNumber: aSequenceNumber

	^ self new initializeWithRadio: aRadio andSequenceNumber: aSequenceNumber ! !

!MockCommandSender methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 18:27'!
initializeWithRadio: aRadio andSequenceNumber: aSequenceNumber 

	radio := aRadio .
	nextSequenceNumber := aSequenceNumber ! !

Object subclass: #CommandSerializer
	instanceVariableNames: 'commandToSerialize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-SendingCommands'!

!CommandSerializer class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:36'!
for: aCommandCall 
	
	^ self new initializeWithCommandToSerialize: aCommandCall ! !

!CommandSerializer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:36'!
initializeWithCommandToSerialize: aCommandToSerialize
	
	commandToSerialize := aCommandToSerialize ! !

!CommandSerializer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:54'!
serialize: anInputToSerialize as: aCommandInputDefinition
 
	| stream serializedInput|
	stream := ReadStream on: anInputToSerialize .
	serializedInput := #[] .
	aCommandInputDefinition do: [ :aSatelliteDataType | | serializationFormula |
		serializationFormula := SerializationFormulas new lookFor: aSatelliteDataType  .
		serializedInput := serializedInput , (serializationFormula value: stream next) ] .
	
	^ serializedInput ! !

!CommandSerializer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:36'!
value

	^ commandToSerialize accept: self! !

!CommandSerializer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:32'!
visitCommandCall: aCommandCall 
	
	| serializedSubSystemId serializedCommandId commandInput serializedInput |
	serializedSubSystemId := aCommandCall subSystem id asByteArrayOfSize: 1 .
	serializedCommandId := aCommandCall command id asByteArrayOfSize: 1 .
	
	commandInput := aCommandCall command input .
	serializedInput := self serialize: aCommandCall inputForTheCommand as: commandInput .

	^ serializedSubSystemId , serializedCommandId , serializedInput! !

!CommandSerializer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:34'!
visitCompoundCommandCall: aCompoundCommandCall 
	
	| serializedCompoundCommand |
	serializedCompoundCommand := #[] .
	aCompoundCommandCall commandCalls do: [ :aCommandCall | 
		serializedCompoundCommand:= serializedCompoundCommand , (aCommandCall accept: self) ] .
	^ serializedCompoundCommand! !

Object subclass: #CommandTicket
	instanceVariableNames: 'package commandSent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-SendingCommands'!

!CommandTicket class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 18:40'!
for: aCommandCall as: aByteArray 
	
	^ self new initializeWithCommandCall: aCommandCall andPackage: aByteArray ! !

!CommandTicket methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:42'!
commandSent

	^ commandSent! !

!CommandTicket methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:41'!
initializeWithCommandCall: aCommandCall andPackage: aByteArray 
	
	| serializedSequenceNumber |
	commandSent := aCommandCall .
	package := aByteArray .! !

!CommandTicket methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 18:44'!
package

	^ package! !

!CommandTicket methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 18:47'!
sequenceNumber

	| stream |
	stream := ReadStream on: package .
	^ (stream next: 3; next: 3) asInteger ! !

Object subclass: #CompoundCommandCall
	instanceVariableNames: 'commandCalls'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!CompoundCommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:23'!
noCommandsMessage
	
	^ 'A compound command cannot be created without commands'! !

!CompoundCommandCall class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:23'!
with: anOrderedCollection 
	
	anOrderedCollection isEmpty ifTrue: [ self error: self noCommandsMessage ].
	
	^ self new initializeWithCommandCalls: anOrderedCollection ! !

!CompoundCommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:31'!
accept: aVisitor 
	
	^ aVisitor visitCompoundCommandCall: self! !

!CompoundCommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:21'!
commandCalls

	^ commandCalls! !

!CompoundCommandCall methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:21'!
initializeWithCommandCalls: anOrderedCollection 
	
	commandCalls := anOrderedCollection ! !

Object subclass: #ConsoleState
	instanceVariableNames: 'console'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Console'!

!ConsoleState class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 12:10'!
for: aSatelliteConsole 
	
	^ self new initializeWithConsole: aSatelliteConsole ! !

!ConsoleState methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:59'!
= aState

	^ self class = aState class! !

!ConsoleState methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 12:10'!
initializeWithConsole: aSatelliteConsole 
	
	console := aSatelliteConsole ! !

!ConsoleState methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:28'!
listenAnswer: aByteArray 

	self subclassResponsibility ! !

!ConsoleState methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:37'!
sendCommand: aCommandCall 
	
	self subclassResponsibility ! !

ConsoleState subclass: #TurnedOffState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Console'!

!TurnedOffState methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:27'!
listenAnswer: aByteArray 
	
	^ console listenAnswerWhenOff: aByteArray ! !

!TurnedOffState methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 12:12'!
sendCommand: aCommandCall 
	
	^ console sendCommandWhenOff: aCommandCall ! !

ConsoleState subclass: #TurnedOnState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Console'!

!TurnedOnState methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:26'!
listenAnswer: aByteArray 
	
	^ console listenAnswerWhenOn: aByteArray ! !

!TurnedOnState methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 12:11'!
sendCommand: aCommandCall 
	
	^ console sendCommandWhenOn: aCommandCall ! !

Object subclass: #Formulas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Formulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:28'!
lookFor: aSatelliteDataType 
	
	^ aSatelliteDataType accept: self! !

!Formulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:29'!
visitArrayOfFixedSize: anArrayOfFixedSize 

	self subclassResponsibility ! !

!Formulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:29'!
visitEnum: anEnum

	self subclassResponsibility ! !

!Formulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:29'!
visitSatelliteString: aSatelliteString 

	self subclassResponsibility ! !

!Formulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:29'!
visitSignedInteger: aSignedInteger 
	
	self subclassResponsibility ! !

!Formulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:29'!
visitStringWithSize: aStringWithSize

	self subclassResponsibility ! !

!Formulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:28'!
visitUnsignedInteger: anUnsignedInteger 
	
	self subclassResponsibility ! !

Formulas subclass: #ParsingFormulas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!ParsingFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:47'!
visitArrayOfFixedSize: anArrayOfFixedSize 

	^ [ :aStream |
		(1 to: anArrayOfFixedSize size) collect: [ :anIndex |
			(ParsingFormulas new lookFor: anArrayOfFixedSize dataType) value: aStream  
			]
		]! !

!ParsingFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:44'!
visitEnum: anEnum

	^ [ :aStream | | labelValue |
		labelValue := (ParsingFormulas new lookFor: Unsigned8BitInteger new) value: aStream .
		[anEnum labelAt: labelValue] on: Error do: [ ParsingError new signal ]
		] .! !

!ParsingFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 23:15'!
visitSatelliteString: aSatelliteString 
	
	^ [ :aStream | aStream upToEnd asString ]! !

!ParsingFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 23:14'!
visitSignedInteger: aSignedInteger

	^ [ :aStream | | byteArrayToTranslate integerFromByteArray |
		byteArrayToTranslate := aStream next: aSignedInteger bytes .
		(byteArrayToTranslate byteSize < aSignedInteger bytes) 
			ifTrue: [ ^ ParsingError new signal ] .
		integerFromByteArray := byteArrayToTranslate asInteger .
		( integerFromByteArray > aSignedInteger max )
			ifTrue: [  integerFromByteArray - ((aSignedInteger max + 1) * 2) ]
			ifFalse: [ integerFromByteArray ]
		]! !

!ParsingFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:42'!
visitStringWithSize: aStringWithSize

	^ [ :aStream | | stringSize parsedString |
		stringSize := (ParsingFormulas new lookFor: aStringWithSize sizeRepresentation) value: aStream . 
		parsedString := (aStream next: stringSize) asString .
		parsedString size ~= stringSize ifTrue: [ ^ ParsingError new signal ] .
		parsedString .
		]! !

!ParsingFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 23:08'!
visitUnsignedInteger: anUnsignedInteger

	^ [ :aStream | | byteArrayToTranslate |
		byteArrayToTranslate := aStream next: anUnsignedInteger bytes .
		(byteArrayToTranslate byteSize < (anUnsignedInteger bytes)) ifTrue: [ ^ ParsingError new signal ] .
		byteArrayToTranslate asInteger ]! !

Formulas subclass: #SerializationFormulas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!SerializationFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 13:05'!
twoComplement: integer size: size 
       
	| mask | 

	mask := (2 raisedTo: size) - 1. 
	^ mask bitAnd: ((integer abs bitXor: mask) + 1)  ! !

!SerializationFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 17:41'!
visitArrayOfFixedSize: anArrayOfFixedSize 
	
	^ [ :aCollection | | serializedAnswer |
		serializedAnswer := #[].
		aCollection do: [ :anElement | | arrayDataTypeFormula |
			arrayDataTypeFormula := SerializationFormulas new lookFor: anArrayOfFixedSize dataType . 
			serializedAnswer := serializedAnswer , (arrayDataTypeFormula value: anElement) 
			] .
		serializedAnswer 
		]! !

!SerializationFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 17:35'!
visitEnum: anEnum 
	
	^ [ :aLabel | (SerializationFormulas new lookFor: Unsigned8BitInteger new) value: (anEnum valueAt: aLabel) ]! !

!SerializationFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 17:21'!
visitSatelliteString: aSatelliteString 
	
	^ [ :aString | aString asByteArray ]! !

!SerializationFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 13:05'!
visitSignedInteger: aSignedInteger
	
	^ [ :anInteger |
		anInteger negative 
			ifFalse: [ anInteger asByteArrayOfSize: aSignedInteger bytes ]
			ifTrue: [ (self twoComplement: anInteger size: aSignedInteger bitsUsed) asByteArrayOfSize: aSignedInteger bytes ]  ]! !

!SerializationFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 17:24'!
visitStringWithSize: aSizedString 
	
	^ [ :aString | | serializedSize |
		serializedSize := (SerializationFormulas new lookFor: aSizedString sizeRepresentation) value: aString size .
		serializedSize , aString asByteArray  ]! !

!SerializationFormulas methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:57'!
visitUnsignedInteger: anUnsignedInteger 
	
	^ [ :anInteger | anInteger asByteArrayOfSize: anUnsignedInteger bytes ]! !

Object subclass: #IdAndNameIdentifier
	instanceVariableNames: 'id name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!IdAndNameIdentifier class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:45'!
with: anId with: aName 
	
	^ self new initializeWithId: anId withName: aName ! !

!IdAndNameIdentifier methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:46'!
= anIdentifier

	^ id = anIdentifier id or: (name = anIdentifier name)! !

!IdAndNameIdentifier methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:46'!
id

	^ id! !

!IdAndNameIdentifier methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:46'!
initializeWithId: anId withName: aName 

	id := anId .
	name := aName ! !

!IdAndNameIdentifier methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 09:46'!
name

	^ name! !

Object subclass: #IdentifiedWithIdAndName
	instanceVariableNames: 'identifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

IdentifiedWithIdAndName subclass: #CommandDefinition
	instanceVariableNames: 'input output'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!CommandDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:04'!
invalidInputMessage 

	^ 'Invalid input for the command'! !

!CommandDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:04'!
invalidOutputMessage
	
	^ 'Invalid output for the command'! !

!CommandDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:04'!
withId: anId name: aName input: anInput output: aCommandOutput 
	
	(aCommandOutput isKindOf: CommandOutput) 
		ifFalse: [ self error: self invalidOutputMessage ].
	(anInput allSatisfy: [ :anElement | anElement isKindOf: SatelliteDataType ])
		ifFalse: [ self error: self invalidInputMessage ].
	
	^ self new initializeWithId: anId withName: aName withInput: anInput withAnswer: aCommandOutput ! !

!CommandDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:06'!
= aCommandDefinition 

	^ self name = aCommandDefinition name and:
		(self id = aCommandDefinition id and:
			(self input = aCommandDefinition input and:
				(self output = aCommandDefinition output )))! !

!CommandDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:03'!
initializeWithId: anId withName: aName withInput: anOrderedCollection withAnswer: aCommandAnswerDefinition

	self initializeId: anId andName: aName .
	input := anOrderedCollection .
	output := aCommandAnswerDefinition ! !

!CommandDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:48'!
input

	^ input ! !

!CommandDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:05'!
output

	^ output ! !

!IdentifiedWithIdAndName methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:47'!
id

	^ identifier id! !

!IdentifiedWithIdAndName methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:47'!
identifier 

	^ identifier ! !

!IdentifiedWithIdAndName methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:46'!
initializeId: anId andName: aName 

	identifier := IdAndNameIdentifier with: anId with: aName .! !

!IdentifiedWithIdAndName methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:47'!
name

	^ identifier name! !

IdentifiedWithIdAndName subclass: #SubSystemDefinition
	instanceVariableNames: 'commandsManual'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!SubSystemDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:14'!
withId: anId name: aName definingCommands: aCollectionOfCommandDefinitions

	| commandsManual |
	commandsManual := CommandsManual for: aCollectionOfCommandDefinitions .
	
	^ self new initializeWithId: anId withName: aName withCommands: commandsManual  ! !

!SubSystemDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:25'!
= aSubSystemDefinition 

	^ self name = aSubSystemDefinition name and:
		( self id = aSubSystemDefinition id and:
			(self commands = aSubSystemDefinition commands))! !

!SubSystemDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:58'!
command: aName 
	
	^ commandsManual at: aName ! !

!SubSystemDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:29'!
commands

	^ commandsManual values! !

!SubSystemDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:45'!
initializeWithId: anId withName: aName withCommands: aCollectionOfCommandDefinitions 

	self initializeId: anId andName: aName .
	commandsManual := aCollectionOfCommandDefinitions ! !

Object subclass: #IdentifiedWithIdAndNameDictionary
	instanceVariableNames: 'dictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

IdentifiedWithIdAndNameDictionary subclass: #CommandsManual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!CommandsManual class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:35'!
twoDefinitionsWithTheSameIdentifierMessage
	
	^ 'Cannot have two commands with the same name or id'! !

!IdentifiedWithIdAndNameDictionary class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:33'!
for: aCollectionOfDefinitions 

	(self twoDefinitionsWithTheSameIdentifier: aCollectionOfDefinitions )
		ifTrue: [ self error: self twoDefinitionsWithTheSameIdentifierMessage ] .

	^ self new initializeWithDefinitions: aCollectionOfDefinitions ! !

!IdentifiedWithIdAndNameDictionary class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:47'!
twoDefinitionsWithTheSameIdentifier: aCollectionOfDefinitions
	
	| identifiers |
	identifiers := aCollectionOfDefinitions collect: [ :aDefinition | aDefinition identifier ] .
	^ identifiers anySatisfy: [ :anIdentifier | (identifiers occurrencesOf: anIdentifier) > 1]! !

!IdentifiedWithIdAndNameDictionary class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:21'!
twoDefinitionsWithTheSameIdentifierMessage

	self subclassResponsibility! !

!IdentifiedWithIdAndNameDictionary methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:06'!
= anIdentifiedWithIdAndNameDictionary

	self halt.
	^ self class = anIdentifiedWithIdAndNameDictionary class and:
		( self size = anIdentifiedWithIdAndNameDictionary size and:
			( self values = anIdentifiedWithIdAndNameDictionary values))! !

!IdentifiedWithIdAndNameDictionary methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:59'!
at: aName

	^ dictionary at: aName ! !

!IdentifiedWithIdAndNameDictionary methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:27'!
initializeWithDefinitions: aCollection  
	
	dictionary := Dictionary new .
	aCollection 
		do: [ :anElement | dictionary at: anElement name put: anElement ]! !

!IdentifiedWithIdAndNameDictionary methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:27'!
size

	^ dictionary size! !

!IdentifiedWithIdAndNameDictionary methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:29'!
values

	^ dictionary values! !

IdentifiedWithIdAndNameDictionary subclass: #SubSystemManual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!SubSystemManual class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:21'!
twoDefinitionsWithTheSameIdentifierMessage
	
	^ 'Cannot have two subsystems with the same name or id'! !

Object subclass: #MacGenerator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-SendingCommands'!

MacGenerator subclass: #MD5MacGenerator
	instanceVariableNames: 'sequenceNumber serializedCommandCall'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-SendingCommands'!

!MD5MacGenerator class methodsFor: 'key' stamp: 'NahuelGarbezza 5/18/2013 09:41'!
key

	"
	'Generate a random binary key' asByteArray adding 16r00 up to 32 len
	"
	
	^ #[71 101 110 101 114 97 116 101 32 97 32 114 97 110 100 111 109 32 98 105 110 97 114 121 32 107 101 121 00 00 00 00]! !

!MD5MacGenerator methodsFor: 'initialization' stamp: 'HernanWilkinson 3/25/2013 14:54'!
initializeFor: aSerializedCommandCall with: aSequenceNumber

	serializedCommandCall := aSerializedCommandCall.
	sequenceNumber := aSequenceNumber.
! !

!MD5MacGenerator methodsFor: 'evaluating - private' stamp: 'HernanWilkinson 3/25/2013 15:36'!
macFrom: md5

	| mac |
	
	mac := md5 first: 3.
	mac swap: 1 with: 3.
	^ mac! !

!MD5MacGenerator methodsFor: 'evaluating - private' stamp: 'NahuelGarbezza 5/18/2013 09:40'!
md5For: aStream

	aStream reset.
	^ MD5 hashStream: aStream! !

!MD5MacGenerator methodsFor: 'evaluating - private' stamp: 'NahuelGarbezza 5/18/2013 09:41'!
streamToCalculateMD5

	| stream |
	stream := ReadWriteStream on: ByteArray new.
	stream
		nextPutAll: self key;
		nextPut: (sequenceNumber bitAnd: 16rFF);
		nextPut: ((sequenceNumber digitAt: 2) bitAnd: 16rFF);
		nextPut: ((sequenceNumber digitAt: 3) bitAnd: 16rFF);
		nextPut: ((sequenceNumber digitAt: 4) bitAnd: 16rFF);
		nextPutAll: serializedCommandCall.
	^ stream! !

!MD5MacGenerator methodsFor: 'evaluating' stamp: 'HernanWilkinson 3/25/2013 15:21'!
value

	| stream md5 mac |
	
	stream := self streamToCalculateMD5.
	md5 := self md5For: stream.
	mac := self macFrom: md5.
	
	^ mac
	! !

!MacGenerator class methodsFor: 'instance creation' stamp: 'NahuelGarbezza 5/18/2013 09:41'!
for: aSerializedCommandCall with: aSequenceNumber

	^ self new initializeFor: aSerializedCommandCall with: aSequenceNumber! !

!MacGenerator class methodsFor: 'key' stamp: 'HernanWilkinson 3/25/2013 14:57'!
key

	self subclassResponsibility ! !

!MacGenerator methodsFor: 'initialization' stamp: 'HernanWilkinson 3/25/2013 14:53'!
initializeFor: aSerializedCommandCall with: aSequenceNumber 

	self subclassResponsibility ! !

!MacGenerator methodsFor: 'evaluating' stamp: 'NahuelGarbezza 5/18/2013 09:40'!
key

	^ self class key! !

!MacGenerator methodsFor: 'evaluating' stamp: 'HernanWilkinson 3/25/2013 14:53'!
value

	self subclassResponsibility ! !

Object subclass: #ObjectFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Tests'!

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 18:03'!
aNameCommandDefinition
	
	^ CommandDefinition 
		withId: 0 
		name: #name 
		input: #() 
		output: (OutputParameter of: SatelliteString new named: #name )! !

!ObjectFactory methodsFor: 'Platform SubSystem' stamp: 'TomasRodriguez 10/31/2013 18:03'!
aPlatformResetCommandDefinition
	
	^ CommandDefinition
		withId: 33
		name: #reset 
		input: #()
		output: Void new! !

!ObjectFactory methodsFor: 'Platform SubSystem' stamp: 'TomasRodriguez 10/31/2013 18:03'!
aPlatformSetModeModeCommandDefinition
	
	^ CommandDefinition
		withId: 32
		name: #setModeMode: 
		input: (OrderedCollection with: Unsigned8BitInteger new)
		output: Void new! !

!ObjectFactory methodsFor: 'Platform SubSystem' stamp: 'TomasRodriguez 10/31/2013 18:03'!
aPlatformShortTelemetryCommandDefinition
	
	^ CommandDefinition
		withId: 2
		name: #short_telemetry
		input: #()
		output: Void new! !

!ObjectFactory methodsFor: 'Platform SubSystem' stamp: 'TomasRodriguez 10/30/2013 10:03'!
aPlatformSubSystemDefinition
	
	^ SubSystemDefinition withId: 0 name: #PLATFORM definingCommands: self validPlatformCommands .! !

!ObjectFactory methodsFor: 'Platform SubSystem' stamp: 'TomasRodriguez 10/31/2013 18:03'!
aPlatformTelemetryCommandDefinition
	
	| answer |
	answer := OrderedCollection new .
	answer add: (OutputParameter of: Unsigned64BitInteger new named: 'tickCount' ) .
	answer add: (OutputParameter of: Unsigned8BitInteger new named: 'currentMode' ) .
	
	^ CommandDefinition
		withId: 1
		name: #telemetry
		input: #()
		output: (Struct with: answer)! !

!ObjectFactory methodsFor: 'Platform SubSystem' stamp: 'TomasRodriguez 10/31/2013 18:03'!
aPlatformVersionCommandDefinition
	
	^ CommandDefinition
		withId: 34
		name: #version 
		input: #()
		output: (OutputParameter of: SatelliteString new named: 'version')! !

!ObjectFactory methodsFor: 'Thermal SubSystem' stamp: 'TomasRodriguez 10/31/2013 18:03'!
aThermalSetMatrixKeyKeyCommandDefinition
	
	^ CommandDefinition
		withId: 32
		name: #setMatrixKeyKey:
		input: (OrderedCollection with: Unsigned32BitInteger new)
		output: Void new! !

!ObjectFactory methodsFor: 'Thermal SubSystem' stamp: 'TomasRodriguez 10/31/2013 18:03'!
aThermalShortTelemetryCommandDefinition
	
	^ CommandDefinition
		withId: 2
		name: #short_telemetry
		input: #()
		output: (OutputParameter of: Signed8BitInteger new named: 'battery_temp' )! !

!ObjectFactory methodsFor: 'Thermal SubSystem' stamp: 'TomasRodriguez 10/30/2013 10:02'!
aThermalSubSystemDefinition
	
	^ SubSystemDefinition withId: 3 name: #THERMAL definingCommands: self validThermalCommands .! !

!ObjectFactory methodsFor: 'Thermal SubSystem' stamp: 'TomasRodriguez 10/31/2013 18:03'!
aThermalTelemetryCommandDefinition
	
	| answer |
	answer := OrderedCollection new .
	answer add: (OutputParameter of: Signed16BitInteger new named: 'battery_temp' ) .
	answer add: (OutputParameter of: Signed16BitInteger new named: 'radio_temp' ) .
	
	^ CommandDefinition
		withId: 1
		name: #telemetry
		input: #()
		output: (Struct with: answer )! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:20'!
subSystemWithoutBasicCommands
	
	^ SubSystemDefinition withId: 1 name: #invalid definingCommands: #()! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 10:00'!
validBeaconAnswer
	
	^ SatelliteAnswer for: #[255 255 240 0 0 0 0 0 0 0 25 90 255 255 0 1] .! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 20:47'!
validParsedBeacon
	
	^ (SignalParser for: self validBeaconAnswer from: self validSatelliteDefinition) value ! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:14'!
validParsedCommandAnswerForThermalName
	| commandAnswer |
	commandAnswer := self validThermalNameSatelliteAnswer.
	^ (CommandAnswerParser for: commandAnswer from: self validThermalNameCommandCall) value! !

!ObjectFactory methodsFor: 'Platform SubSystem' stamp: 'TomasRodriguez 10/30/2013 10:04'!
validPlatformCommands
	
	| commands |
	commands := OrderedCollection new. 
	^ commands	
		add: self aNameCommandDefinition ;
		add: self aPlatformTelemetryCommandDefinition ;
		add: self aPlatformShortTelemetryCommandDefinition ;
		add: self aPlatformSetModeModeCommandDefinition ;
		add: self aPlatformResetCommandDefinition ;
		add: self aPlatformVersionCommandDefinition ;
		yourself ! !

!ObjectFactory methodsFor: 'Satellite' stamp: 'TomasRodriguez 10/30/2013 10:55'!
validSatelliteDefinition
	
	^ SatelliteDefinition composedOf: self validSubSystemsForASatellite .! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:29'!
validShortBeaconAnswer
	
	^ SatelliteAnswer for: #[255 255 243 255]! !

!ObjectFactory methodsFor: 'Satellite' stamp: 'TomasRodriguez 10/30/2013 10:03'!
validSubSystemsForASatellite
	
	| subSystems |
	subSystems := OrderedCollection new. 
	^ subSystems	
		add: self aThermalSubSystemDefinition ;
		add: self aPlatformSubSystemDefinition ;
		yourself ! !

!ObjectFactory methodsFor: 'Thermal SubSystem' stamp: 'TomasRodriguez 10/30/2013 09:59'!
validThermalCommands
	
	| commands |
	commands := OrderedCollection new. 
	^ commands	
		add: self aNameCommandDefinition ;
		add: self aThermalTelemetryCommandDefinition ;
		add: self aThermalShortTelemetryCommandDefinition ;
		add: self aThermalSetMatrixKeyKeyCommandDefinition ;
		yourself ! !

!ObjectFactory methodsFor: 'Command Calls' stamp: 'TomasRodriguez 10/30/2013 12:22'!
validThermalNameCommandCall
	
	^ CommandCall 
		for: self aNameCommandDefinition 
		from:  self aThermalSubSystemDefinition 
		in: self validSatelliteDefinition 
		withInputParameters: #()! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:02'!
validThermalNameCommandTicket
	
	| commandCall serializedCommandCall serializedSequenceNumber serializedMac |
	commandCall := self validThermalNameCommandCall .
	serializedCommandCall := (CommandSerializer for: commandCall) value .
	serializedSequenceNumber := 16r1 asByteArrayOfSize: 3 .
	serializedMac := (MD5MacGenerator for: serializedCommandCall with: 16r1) value .
	^ CommandTicket for: commandCall as: (serializedMac, serializedSequenceNumber, serializedCommandCall )! !

!ObjectFactory methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:14'!
validThermalNameSatelliteAnswer
	^ SatelliteAnswer for: #[0 0 1 84 72 69 82 77 65 76]! !

!ObjectFactory methodsFor: 'Command Calls' stamp: 'TomasRodriguez 10/30/2013 12:38'!
validThermalSetMatrixKeyKeyCommandCallWithInput: anInput 
	
	^ CommandCall 
		for: self aThermalSetMatrixKeyKeyCommandDefinition  
		from:  self aThermalSubSystemDefinition 
		in: self validSatelliteDefinition 
		withInputParameters: anInput! !

!ObjectFactory methodsFor: 'Command Calls' stamp: 'TomasRodriguez 10/31/2013 10:43'!
validThermalTelemetryCommandCall
	
	^ CommandCall 
		for: self aThermalTelemetryCommandDefinition  
		from:  self aThermalSubSystemDefinition 
		in: self validSatelliteDefinition 
		withInputParameters: #()! !

Object subclass: #ParsedAnswer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

ParsedAnswer subclass: #EmptyAnswer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!EmptyAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:22'!
= anEmptyAnswer

	^ self class = anEmptyAnswer class! !

!ParsedAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:17'!
= aParsedAnswer

	self subclassResponsibility ! !

ParsedAnswer subclass: #ParsedOutputParameter
	instanceVariableNames: 'value name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!ParsedOutputParameter class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:58'!
with: aParsedValue named: aName

	^ self new initializeWithValue: aParsedValue withName: aName ! !

!ParsedOutputParameter methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:18'!
= aParsedValue 

	^ self class = aParsedValue class and:
		(self name = aParsedValue name and:
			(self value = aParsedValue value))! !

!ParsedOutputParameter methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 08:59'!
initializeWithValue: aParsedValue withName: aName 

	value := aParsedValue .
	name := aName ! !

!ParsedOutputParameter methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:01'!
name

	^ name! !

!ParsedOutputParameter methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 09:01'!
value

	^ value ! !

ParsedAnswer subclass: #ParsedSignal
	instanceVariableNames: 'subSystemsAnswersDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!ParsedSignal class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 10:53'!
with: aDictionary 
	
	^ self new initializeWithDictionary: aDictionary ! !

!ParsedSignal methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:19'!
= aParsedSignal 

	^ self class = aParsedSignal class and:
		(self subSystemsAnswersDictionary = aParsedSignal subSystemsAnswersDictionary)! !

!ParsedSignal methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:26'!
informationOf: aSubSystemDefinition 
	
	"No se por que si hago el at: directamente no funciona cuando corro el test, pero si cuando lo debugeo.. rarisimo"
	
	| subSystem|
	subSystem := subSystemsAnswersDictionary keys detect: [ :aSubSystem | aSubSystem identifier = aSubSystemDefinition identifier ] .
	^ subSystemsAnswersDictionary at: subSystem ! !

!ParsedSignal methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 10:54'!
initializeWithDictionary: aDictionary 
	
	subSystemsAnswersDictionary := aDictionary ! !

!ParsedSignal methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:20'!
subSystemsAnswersDictionary

	^ subSystemsAnswersDictionary! !

ParsedAnswer subclass: #ParsedStruct
	instanceVariableNames: 'parsedAnswerParameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!ParsedStruct class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 10:36'!
with: anOrderedCollection 

	^ self new initializeWithParsedAnswerParameters: anOrderedCollection ! !

!ParsedStruct methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:21'!
= aParsedStruct

	^ self class = aParsedStruct class and:
		(self parsedParameters = aParsedStruct parsedParameters )! !

!ParsedStruct methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 10:36'!
initializeWithParsedAnswerParameters: anOrderedCollection 

	parsedAnswerParameters := anOrderedCollection ! !

!ParsedStruct methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 10:38'!
parsedParameters
	
	^ parsedAnswerParameters ! !

Object subclass: #Record
	instanceVariableNames: 'satelliteAnswer parsedAnswer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Console'!

Record subclass: #CommandAnswerRecord
	instanceVariableNames: 'commandCall'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Console'!

!CommandAnswerRecord class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:26'!
of: aParsedCommandAnswer fromSatelliteAnswer: aCommandAnswer withCommandCall: aCommandCall 
	
	^ self new initializeWithParsedAnswer: aParsedCommandAnswer withSatelliteAnswer: aCommandAnswer WithCommandCall: aCommandCall ! !

!CommandAnswerRecord methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:27'!
commandCall

	^ commandCall! !

!CommandAnswerRecord methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:27'!
initializeWithParsedAnswer: aParsedCommandAnswer withSatelliteAnswer: aSatelliteAnswer WithCommandCall: aCommandCall 
	
	self initializeWithParsedAnswer: aParsedCommandAnswer withSatelliteAnswer:  aSatelliteAnswer .
	commandCall := aCommandCall ! !

!Record methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:23'!
initializeWithParsedAnswer: aParsedAnswer withSatelliteAnswer: aSatelliteAnswer 
	
	parsedAnswer := aParsedAnswer .
	satelliteAnswer := aSatelliteAnswer! !

!Record methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:24'!
parsedAnswer
	
	^ parsedAnswer! !

!Record methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:24'!
satelliteAnswer

	^ satelliteAnswer! !

Record subclass: #SignalRecord
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Console'!

!SignalRecord class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:24'!
of: aParsedSignal fromSatelliteAnswer: aSignalAnswer 
	
	^ self new initializeWithParsedAnswer: aParsedSignal withSatelliteAnswer: aSignalAnswer ! !

Object subclass: #SatelliteAnswer
	instanceVariableNames: 'package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

SatelliteAnswer subclass: #CommandAnswer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!CommandAnswer class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 22:07'!
isIdentifiedWith: anInteger 
	
	^ anInteger between: 16r1 and: 16rFFFFF0! !

!CommandAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:36'!
accept: aVisitor
	
	^ aVisitor visitCommandAnswer: self! !

!CommandAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:43'!
sequenceNumber 

	^ ((ReadStream on: package) next: 3) asInteger  ! !

!SatelliteAnswer class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:14'!
for: aByteArray 
	
	| messageIdentifier |
	messageIdentifier := ((ReadStream on: aByteArray ) next: 3) asInteger .

	(Beacon isIdentifiedWith: messageIdentifier) ifTrue: [ ^ Beacon new initializeWithPackage: aByteArray ] .
	(ShortBeacon isIdentifiedWith: messageIdentifier) ifTrue: [ ^ ShortBeacon new initializeWithPackage: aByteArray ] .
	(CommandAnswer isIdentifiedWith: messageIdentifier) ifTrue: [ ^ CommandAnswer new initializeWithPackage: aByteArray ] .
	^ UnknownAnswer new initializeWithPackage: aByteArray ! !

!SatelliteAnswer class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 22:07'!
isIdentifiedWith: anInteger 
	
	self subclassResponsibility ! !

!SatelliteAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:15'!
= aSatelliteAnswer 

	^ self class = aSatelliteAnswer class and:
		(self package = aSatelliteAnswer package)! !

!SatelliteAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:46'!
accept: aVisitor 

	self subclassResponsibility ! !

!SatelliteAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:14'!
initializeWithPackage: aByteArray 
	
	package := aByteArray ! !

!SatelliteAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:14'!
package

	^ package ! !

SatelliteAnswer subclass: #SignalAnswer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

SignalAnswer subclass: #Beacon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!Beacon class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 22:07'!
isIdentifiedWith: anInteger 
	
	^ anInteger = 16rFFFFF0! !

!Beacon methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:33'!
commandAssociated
	
	^ #telemetry! !

SignalAnswer subclass: #ShortBeacon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!ShortBeacon class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 22:07'!
isIdentifiedWith: anInteger 
	
	^ anInteger = 16rFFFFF3! !

!ShortBeacon methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:32'!
commandAssociated
	
	^ #short_telemetry! !

!SignalAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:08'!
accept: aVisitor 
	
	^ aVisitor visitSignalAnswer: self! !

!SignalAnswer methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:35'!
commandAssociated
	
	self subclassResponsibility ! !

SatelliteAnswer subclass: #UnknownAnswer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!UnknownAnswer class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 22:19'!
isIdentifiedWith: anInteger 
	
	| otherSatelliteAnswers |
	otherSatelliteAnswers := (SatelliteAnswer subclasses reject: [ :aSatelliteAnswer | aSatelliteAnswer = self ]) .
	^ (otherSatelliteAnswers anySatisfy: [ :aSatelliteAnswer | aSatelliteAnswer isIdentifiedWith: anInteger ]) not! !

Object subclass: #SatelliteAnswerParser
	instanceVariableNames: 'satelliteAnswer parsedAnswer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

SatelliteAnswerParser subclass: #CommandAnswerParser
	instanceVariableNames: 'commandCall sequenceNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!CommandAnswerParser class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:13'!
for: aCommandAnswer from: aCommandCall 
	
	^ self new initializeWithParsedAnswer: aCommandAnswer withCommandCall: aCommandCall ! !

!CommandAnswerParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:08'!
initializeWithParsedAnswer: aCommandAnswer withCommandCall: aCommandCall 
	
	satelliteAnswer := aCommandAnswer .
	commandCall := aCommandCall ! !

!CommandAnswerParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:39'!
parseSatelliteAnswer

	| stream |
	stream := ReadStream on: satelliteAnswer package .
	sequenceNumber := (stream next: 3) asInteger  .
	^ (CommandOutputParser for: commandCall command output as: stream ) value! !

!SatelliteAnswerParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:30'!
parseSatelliteAnswer

	self subclassResponsibility ! !

!SatelliteAnswerParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:09'!
value

	parsedAnswer isNil ifTrue: [ parsedAnswer := self parseSatelliteAnswer ] .
	^ parsedAnswer! !

SatelliteAnswerParser subclass: #SignalParser
	instanceVariableNames: 'satelliteDefinition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-ParsingAnswers'!

!SignalParser class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:31'!
for: aSignal from: aSatelliteDefinition 
	
	^ self new initializeWithSignal: aSignal withSatelliteDefinition: aSatelliteDefinition ! !

!SignalParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 21:08'!
initializeWithSignal: aSignal withSatelliteDefinition: aSatelliteDefinition 
	
	satelliteAnswer := aSignal .
	satelliteDefinition := aSatelliteDefinition ! !

!SignalParser methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:15'!
parseSatelliteAnswer
	
	| orderedByIdSubSystems answer stream |
	orderedByIdSubSystems := satelliteDefinition subSystems sort: [ :aSubSystem :otherSubSystem | aSubSystem id < otherSubSystem id] .
	answer := Dictionary new .
	stream := ReadStream on: satelliteAnswer package .
	stream next: 3 .
	orderedByIdSubSystems do: [ :aSubSystem | | commandAssociated parser |
		commandAssociated := aSubSystem command: satelliteAnswer commandAssociated .
		parser := CommandOutputParser for: commandAssociated output as: stream .
		answer at: aSubSystem put: parser value
		] .
	^ ParsedSignal with: answer ! !

Object subclass: #SatelliteConsole
	instanceVariableNames: 'satellite radio state records pendingCommandAnswers commandSender'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Console'!

!SatelliteConsole class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:55'!
for: aSatelliteDefinition usingRadio: aRadio
	
	^ self new initializeWithSatelliteDefinition: aSatelliteDefinition withRadio: aRadio ! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 12:13'!
consoleIsOffMessage
	
	^ 'The console is turned off'! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:17'!
findCommandTicketInThePendingCommandAnswers: aCommandAnswer
	
	^ pendingCommandAnswers detect: [ :aCommandTicket | 
		aCommandTicket sequenceNumber = aCommandAnswer sequenceNumber ]
		ifNone: [ ^ Error signal ] .! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:37'!
initializeWithSatelliteDefinition: aSatelliteDefinition withRadio: aRadio 

	satellite := aSatelliteDefinition .
	radio := aRadio .
	radio addListener: self .
	state := TurnedOffState for: self .
	records := LinkedList new .
	commandSender := CommandSender sendThrough: radio .
	pendingCommandAnswers := OrderedCollection new! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:26'!
listen: aByteArray 
	
	^ state listenAnswer: aByteArray ! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:27'!
listenAnswerWhenOff: aByteArray 
	
	^  nil! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:51'!
listenAnswerWhenOn: aByteArray 
	
	| satelliteAnswer parsedAnswer record |
	satelliteAnswer := SatelliteAnswer for: aByteArray .
	[record := satelliteAnswer accept: self .
	records addFirst: record ] on: Error do: []! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:11'!
records
	
	^ records! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:57'!
satellite

	^ satellite! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 12:08'!
send: aCommandCall 
	
	^ state sendCommand: aCommandCall! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 12:13'!
sendCommandWhenOff: aCommandCall 
	
	self error: self consoleIsOffMessage! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:37'!
sendCommandWhenOn: aCommandCall 
	
	| commandTicket |
	commandTicket := commandSender send: aCommandCall .
	pendingCommandAnswers add: commandTicket .
	^ commandTicket ! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:57'!
state

	^ state! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 12:12'!
turnOn
	
	state := TurnedOnState for: self! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 10:43'!
visitCommandAnswer: aCommandAnswer 
	
	| commandTicket commandCall parsedCommand |
	commandTicket := self findCommandTicketInThePendingCommandAnswers: aCommandAnswer .
	commandCall := commandTicket commandSent .
	parsedCommand := (CommandAnswerParser for: aCommandAnswer from: commandCall ) value.
	^ CommandAnswerRecord of: parsedCommand fromSatelliteAnswer: aCommandAnswer withCommandCall: commandCall! !

!SatelliteConsole methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 11/1/2013 09:10'!
visitSignalAnswer: aSignalAnswer 
	
	| parser parsedAnswer |
	parser := SignalParser for: aSignalAnswer from: satellite .
	parsedAnswer := parser value .
	^ SignalRecord of: parsedAnswer fromSatelliteAnswer: aSignalAnswer! !

Object subclass: #SatelliteDataType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

SatelliteDataType subclass: #ArrayOfFixedSize
	instanceVariableNames: 'dataType size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!ArrayOfFixedSize class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 21:14'!
of: aSatelliteDataType withSize: aSize 
	
	^ self new initializeWithDataType: aSatelliteDataType andSize: aSize ! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:02'!
= anArrayUpToEndOfPackage 

	^ self class = anArrayUpToEndOfPackage class and:
		(self size = anArrayUpToEndOfPackage size and:
			(self dataType = anArrayUpToEndOfPackage dataType ))! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 17:40'!
accept: aVisitor 
	
	^ aVisitor visitArrayOfFixedSize: self! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:49'!
canBeRepresentedWith: aCollection

	aCollection isCollection ifFalse: [ ^ false ] .
	^ aCollection size = size and: ( aCollection allSatisfy: [:anElement | dataType canBeRepresentedWith: anElement ])! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:14'!
dataType

	^ dataType! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:11'!
initializeWithDataType: aSatelliteDataType andSize: aSize 
	
	dataType := aSatelliteDataType .
	size := aSize ! !

!ArrayOfFixedSize methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:24'!
size

	^ size! !

SatelliteDataType subclass: #Enum
	instanceVariableNames: 'labels'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Enum class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 21:01'!
withLabels: anArrayOfLabelds
	
	^ self new initializeWithLabels: anArrayOfLabelds ! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/31/2013 11:04'!
= anEnum 

	^ self class = anEnum class and:
		(self labels = labels size and:
			(self labels allSatisfy: [ :aLabel | (self valueAt: aLabel) = (anEnum valueAt: aLabel) ]))! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 17:35'!
accept: aVisitor  
	
	^ aVisitor visitEnum: self! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:52'!
canBeRepresentedWith: aLabel

	^ labels includes: aLabel ! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:21'!
initializeWithLabels: anArrayOfLabels 
	
	labels := anArrayOfLabels ! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:55'!
labelAt: anInteger

	^ labels at: (anInteger + 1) ifAbsent: [ self error: self valueNotFound ]! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:22'!
labels

	^ labels ! !

!Enum methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:23'!
valueAt: aLabel

	^ (labels indexOf: aLabel ifAbsent: [ self error: self labelNotFound ]) -1! !

!SatelliteDataType methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/29/2013 21:23'!
= aSatelliteInteger

	^ self class = aSatelliteInteger class! !

!SatelliteDataType methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/19/2013 11:28'!
accept: aVisitor 

	self subclassResponsibility ! !

!SatelliteDataType methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:37'!
canBeRepresentedWith: anInteger

	self subclassResponsibility ! !

SatelliteDataType subclass: #SatelliteInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!SatelliteInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed

	self subclassResponsibility ! !

!SatelliteInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bytes

	^ self bitsUsed / 8! !

SatelliteInteger subclass: #SignedInteger
	instanceVariableNames: 'min max'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

SignedInteger subclass: #Signed16BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Signed16BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 16! !

SignedInteger subclass: #Signed32BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Signed32BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 32! !

SignedInteger subclass: #Signed64BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Signed64BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 64! !

SignedInteger subclass: #Signed8BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Signed8BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 8! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/28/2013 10:22'!
accept: aVisitor 
	
	^ aVisitor visitSignedInteger: self .! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:35'!
canBeRepresentedWith: anInteger

	anInteger isInteger ifFalse: [ ^ false ] .
	^ anInteger between: min and: max! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
initialize

	| bits |
	bits := self bitsUsed .
	min := (2 raisedTo: bits - 1) negated .
	max := (2 raisedTo: bits - 1) - 1! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 20:34'!
max

	^ max! !

!SignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/18/2013 20:34'!
min

	^ min! !

SatelliteInteger subclass: #UnsignedInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

UnsignedInteger subclass: #Unsigned16BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned16BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	^ 16! !

UnsignedInteger subclass: #Unsigned24BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned24BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 24! !

UnsignedInteger subclass: #Unsigned32BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned32BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 32! !

UnsignedInteger subclass: #Unsigned64BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned64BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 64! !

UnsignedInteger subclass: #Unsigned8BitInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!Unsigned8BitInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:11'!
bitsUsed
	
	^ 8! !

!UnsignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/28/2013 09:44'!
accept: aVisitor 
	
	^ aVisitor visitUnsignedInteger: self .! !

!UnsignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:31'!
canBeRepresentedWith: anInteger

	anInteger isInteger ifFalse: [ ^ false ] .
	^ anInteger > 0 and: ( anInteger < (2 raisedTo: self bitsUsed) )! !

!UnsignedInteger methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 12:46'!
serializationFormula
	
	^ [ :anInteger | anInteger asByteArrayOfSize: self bitsUsed ]! !

SatelliteDataType subclass: #SatelliteString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!SatelliteString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/28/2013 10:22'!
accept: aVisitor 
	
	^ aVisitor visitSatelliteString: self! !

!SatelliteString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:39'!
canBeRepresentedWith: aString

	^ aString isString! !

SatelliteDataType subclass: #SizedString
	instanceVariableNames: 'dataTypeToSerializeSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!SizedString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/28/2013 10:23'!
accept: aVisitor 
	
	^ aVisitor visitStringWithSize: self! !

!SizedString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 11:44'!
canBeRepresentedWith: aString

	aString isString ifFalse: [ ^ false ] .
	^ aString size < (2 raisedTo: self sizeRepresentation bitsUsed )! !

!SizedString methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:19'!
sizeRepresentation
	
	self subclassResponsibility ! !

SizedString subclass: #String16
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!String16 methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:19'!
sizeRepresentation
	
	^ Unsigned16BitInteger new! !

SizedString subclass: #String8
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-DataTypes'!

!String8 methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/21/2013 20:19'!
sizeRepresentation
	
	^ Unsigned8BitInteger new! !

Object subclass: #SatelliteDefinition
	instanceVariableNames: 'subSystemManual'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Satellogic-Structure'!

!SatelliteDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:39'!
allSubSystemsDefineTheBasicCommands: aCollectionOfSubSystemsDefinition

	^ aCollectionOfSubSystemsDefinition allSatisfy: [ :aSubSystem | self subSystemDefineBasicCommands: aSubSystem  ]
	 
	
	! !

!SatelliteDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:41'!
basicCommandsMissingMessage
	
	^ 'Cannot create a satellite with subsystems which does not define the basic commands: "name", "telemetry", "short_telemetry"'! !

!SatelliteDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:33'!
composedOf: aCollectionOfSubSystems 
	
	| aSubSystemManual |
	aSubSystemManual := SubSystemManual for: aCollectionOfSubSystems .
	
	(self allSubSystemsDefineTheBasicCommands: aCollectionOfSubSystems ) 
		ifFalse: [ self error: self basicCommandsMissingMessage ] .
	
	^ self new initializeWithSubSystems: aSubSystemManual ! !

!SatelliteDefinition class methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:39'!
subSystemDefineBasicCommands: aSubSystemDefinition 
	
	|aCollectionOfCommandDefinitions hasTelemetry hasShortTelemetry hasName |
	aCollectionOfCommandDefinitions := aSubSystemDefinition commands .
	
	hasTelemetry := aCollectionOfCommandDefinitions anySatisfy: [ :aCommand | aCommand name = 'telemetry' ] .
	hasShortTelemetry := aCollectionOfCommandDefinitions anySatisfy: [ :aCommand | aCommand name = 'short_telemetry' ] .
	hasName := aCollectionOfCommandDefinitions anySatisfy: [ :aCommand | aCommand name = 'name' ] .
	
	^ hasTelemetry and: (hasShortTelemetry and: (hasName))! !

!SatelliteDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:12'!
initializeWithSubSystems: aSubSystemManual 
	
	subSystemManual := aSubSystemManual ! !

!SatelliteDefinition methodsFor: 'as yet unclassified' stamp: 'TomasRodriguez 10/30/2013 10:30'!
subSystems

	^ subSystemManual values! !
